package cn.ibizlab.codegen.model;

import com.alibaba.fastjson.annotation.JSONField;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonIgnoreProperties;
import lombok.Getter;
import lombok.NoArgsConstructor;
import lombok.Setter;
import lombok.experimental.Accessors;
import net.ibizsys.model.dataentity.defield.IPSDEField;
import org.springframework.util.StringUtils;

import java.util.LinkedHashSet;
import java.util.Set;

@Getter
@Setter
@NoArgsConstructor
@Accessors(chain = true)
@JsonIgnoreProperties(value = "handler")
public class FieldModel extends BaseModel {

    public FieldModel(EntityModel entityModel,IPSDEField deField)
    {
        this.opt=deField;
        this.entity=entityModel;
        this.setCodeName(deField.getCodeName());
        this.setName(deField.getName());
        this.setId(String.format("%1$s-%2$s",entity.getCodeName(),deField.getCodeName()));
    }

    public String getCodeName() {
        return getDataEntityField().getCodeName();
    }

    public String getFieldName() {
        return getDataEntityField().getName();
    }

    public String getLogicName() {
        return getDataEntityField().getLogicName();
    }

    public PropType getType()
    {
        return PropType.findType(this.getDataEntityField().getStdDataType());
    }

    private String expression;


    public IPSDEField getDataEntityField()
    {
        return (IPSDEField)opt;
    }

    private EntityModel entity;

    private RelationshipModel reference;

    private String refFieldName;

    private String refFieldCodeName;

    public boolean isPhisicalDEField()
    {
        return this.getDataEntityField().isPhisicalDEField();
    }

    public boolean isKeyDEField()
    {
        return this.getDataEntityField().isKeyDEField();
    }

    public boolean isLogicValidField()
    {
        return "LOGICVALID".equals(this.getDataEntityField().getPredefinedType());
    }

    public boolean isLastModifyField()
    {
        return "UPDATEDATE".equals(this.getDataEntityField().getPredefinedType());
    }

    public boolean isCreateTimeField()
    {
        return "CREATEDATE".equals(this.getDataEntityField().getPredefinedType());
    }

    public boolean isUnionKeyField()
    {
        return !StringUtils.isEmpty(this.getDataEntityField().getUnionKeyValue());
    }

    public String getDefaultValue()
    {
        if((!StringUtils.isEmpty(getDataEntityField().getDefaultValueType()))||(!StringUtils.isEmpty(getDataEntityField().getDefaultValue())))
        {
            String defaultValue="";
            if(StringUtils.isEmpty(getDataEntityField().getDefaultValueType())||"NONE".equalsIgnoreCase(getDataEntityField().getDefaultValueType()))
            {
                defaultValue=getDataEntityField().getDefaultValue();
            }
            else if(!StringUtils.isEmpty(getDataEntityField().getDefaultValue()))
            {
                defaultValue= String.format("${%1$s.%2$s}",getDataEntityField().getDefaultValueType(),getDataEntityField().getDefaultValue());
            }
            else
            {
                defaultValue= String.format("${%1$s}",getDataEntityField().getDefaultValueType());
            }
            return defaultValue;
        }
        return null;
    }

    public Set<String> getSearchMode()
    {
        Set<String> modes=new LinkedHashSet<>();
        if(getDataEntityField().getAllPSDEFSearchModes()!=null)
        {
            getDataEntityField().getAllPSDEFSearchModes().forEach(item->modes.add(item.getValueOP().toLowerCase()));
        }
        if(getDataEntityField().isEnableQuickSearch())
            modes.add("query");
        return modes;
    }



    public String getStringValue(String key,String defaultVal)
    {
        return extParams.getStringValue(key,defaultVal);
    }

    public String getColumnName(String dsType)
    {
        return this.getStringValue("column-"+dsType.toLowerCase(),getFieldName());
    }





}
